/****************************************************************************
**
** Copyright (C) 2020 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the tests of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

import QtQuick
import QtQuick3D

Item {
    width: 460
    height: 460

    // This exactly same content should appear 4 times, in different ways.
    Node {
        id: sceneRoot
        opacity: 0.8
        PerspectiveCamera {
            position: Qt.vector3d(0, 0, 200)
        }
        DirectionalLight {
        }
        Model {
            source: "#Cube"
            materials: DefaultMaterial {
                diffuseColor: "green"
            }
            eulerRotation: Qt.vector3d(45, 45, 45)
        }
    }

    SceneComponent {
        id: sceneRoot2
        opacity: 0.8
    }

    // View1, importScene with node id of local component + content inside
    View3D {
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.margins: 20
        width: 200
        height: 200
        environment: SceneEnvironment {
            backgroundMode: SceneEnvironment.Color
            clearColor: Qt.rgba(0.5, 0.5, 0.5, 1)
        }
        Component.onCompleted: {
            importScene = sceneRoot;
        }

        Model {
            // Behind, so partly visible
            source: "#Cube"
            scale: Qt.vector3d(0.5, 0.5, 0.5)
            y: 50
            z: -120
            materials: DefaultMaterial {
                diffuseColor: "red"
            }
            eulerRotation: Qt.vector3d(45, 45, 45)
        }
        Model {
            // Above, so fully visible
            source: "#Cube"
            scale: Qt.vector3d(0.1, 0.1, 0.1)
            y: -10
            z: 120
            materials: DefaultMaterial {
                diffuseColor: "blue"
            }
            eulerRotation: Qt.vector3d(45, 45, 0)
        }
    }

    // View2, importScene with node id of external component + content inside
    View3D {
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.margins: 20
        width: 200
        height: 200
        environment: SceneEnvironment {
            backgroundMode: SceneEnvironment.Color
            clearColor: Qt.rgba(0.5, 0.5, 0.5, 1)
        }
        importScene: sceneRoot2
        Component.onCompleted: {
            // Currently this doesn't do anything
            importScene = null;
        }
        Model {
            // Behind, so partly visible
            source: "#Cube"
            scale: Qt.vector3d(0.5, 0.5, 0.5)
            y: 50
            z: -120
            materials: DefaultMaterial {
                diffuseColor: "red"
            }
            eulerRotation: Qt.vector3d(45, 45, 45)
        }
        Model {
            // Above, so fully visible
            source: "#Cube"
            scale: Qt.vector3d(0.1, 0.1, 0.1)
            y: -10
            z: 120
            materials: DefaultMaterial {
                diffuseColor: "blue"
            }
            eulerRotation: Qt.vector3d(45, 45, 0)
        }
    }

    // View3, importScene with external component + content inside
    View3D {
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        anchors.margins: 20
        width: 200
        height: 200
        environment: SceneEnvironment {
            backgroundMode: SceneEnvironment.Color
            clearColor: Qt.rgba(0.5, 0.5, 0.5, 1)
        }
        importScene: SceneComponent {
            id: sceneRoot3
            opacity: 0.8
        }
        Model {
            // Behind, so partly visible
            source: "#Cube"
            scale: Qt.vector3d(0.5, 0.5, 0.5)
            y: 50
            z: -120
            materials: DefaultMaterial {
                diffuseColor: "red"
            }
            eulerRotation: Qt.vector3d(45, 45, 45)
        }
        Model {
            // Above, so fully visible
            source: "#Cube"
            scale: Qt.vector3d(0.1, 0.1, 0.1)
            y: -10
            z: 120
            materials: DefaultMaterial {
                diffuseColor: "blue"
            }
            eulerRotation: Qt.vector3d(45, 45, 0)
        }
    }

    // View4, content inside
    View3D {
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        anchors.margins: 20
        width: 200
        height: 200
        environment: SceneEnvironment {
            backgroundMode: SceneEnvironment.Color
            clearColor: Qt.rgba(0.5, 0.5, 0.5, 1)
        }
        Node {
            id: sceneRoot4
            opacity: 0.8
            PerspectiveCamera {
                position: Qt.vector3d(0, 0, 200)
            }
            DirectionalLight {
            }
            Model {
                source: "#Cube"
                materials: DefaultMaterial {
                    diffuseColor: "green"
                }
                eulerRotation: Qt.vector3d(45, 45, 45)
            }
        }
        Model {
            // Behind, so partly visible
            source: "#Cube"
            scale: Qt.vector3d(0.5, 0.5, 0.5)
            y: 50
            z: -120
            materials: DefaultMaterial {
                diffuseColor: "red"
            }
            eulerRotation: Qt.vector3d(45, 45, 45)
        }
        Model {
            // Above, so fully visible
            source: "#Cube"
            scale: Qt.vector3d(0.1, 0.1, 0.1)
            y: -10
            z: 120
            materials: DefaultMaterial {
                diffuseColor: "blue"
            }
            eulerRotation: Qt.vector3d(45, 45, 0)
        }
    }
}
